"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkDetectors = checkDetectors;
const sqlInjectionDetector_1 = require("../../detectors/sqlInjectionDetector");
const xssDetector_1 = require("../../detectors/xssDetector");
const commandInjectionDetector_1 = require("../../detectors/commandInjectionDetector");
const ldapInjectionDetector_1 = require("../../detectors/ldapInjectionDetector");
const xpathInjectionDetector_1 = require("../../detectors/xpathInjectionDetector");
const fileInclusionDetector_1 = require("../../detectors/fileInclusionDetector");
const alertConstants_1 = require("../constants/alertConstants");
/**
 * Comprueba diferentes detectores de seguridad en un valor dado y agrega los problemas detectados a una lista de problemas.
 * Los detectores incluyen la detección de sentencias SQL, inyección de SQL, XSS (Cross-Site Scripting) e inyección de scripts.
 *
 * @param value - El valor a comprobar.
 * @param key - La clave asociada al valor.
 * @param issues - La lista de problemas detectados.
 * @param options - Las opciones de escaneo.
 * @param isKey - Indica si el valor es una clave.
 * @returns void
 *
 * @remarks
 * Este método comprueba si el valor dado contiene sentencias SQL, inyección de SQL, XSS o inyección de scripts.
 * Si se detecta alguno de estos problemas, se agrega un mensaje de alerta a la lista de problemas.
 *
 * @example
 * checkDetectors("SELECT * FROM users", "query", issues, { detectSentences: true, detectReservedWords: true }, true);
 */
function checkDetectors(value, key, issues, options, isKey = false) {
    const stringValue = JSON.stringify(value);
    const stringKey = JSON.stringify(key);
    const { detectSentences, detectReservedWords } = options;
    const keyOrValue = isKey ? 'key' : stringKey;
    if (detectSentences && (0, sqlInjectionDetector_1.isSQLSentence)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.sqlSentenceDetected(keyOrValue, stringValue));
    }
    if (detectReservedWords && (0, sqlInjectionDetector_1.detectSQLInjection)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.sqlInjectionDetected(keyOrValue, stringValue));
    }
    if ((0, xssDetector_1.detectXSS)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.xssDetected(keyOrValue, stringValue));
    }
    if ((0, commandInjectionDetector_1.detectCommandInjection)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.commandInjectionDetected(keyOrValue, stringValue));
    }
    if ((0, ldapInjectionDetector_1.detectLDAPInjection)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.ldapInjectionDetected(keyOrValue, stringValue));
    }
    if ((0, xpathInjectionDetector_1.detectXPathInjection)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.xpathInjectionDetected(keyOrValue, stringValue));
    }
    if ((0, fileInclusionDetector_1.detectFileInclusion)(stringValue)) {
        issues.push(alertConstants_1.ALERT_MESSAGES.fileInclusionDetected(keyOrValue, stringValue));
    }
}
//# sourceMappingURL=checkDetectors.js.map