import { scanUrl } from './scan/scanUrl';
import { scanBody } from './scan/scanBody';
import { scanHeaders } from './scan/scanHeaders';
import { scanFormData } from './scan/scanFormData';
import { scanCookies } from './scan/scanCookies';
import { ScanOptions } from './interfaces/ITScanOptions';
import { ScanRequestParams } from './interfaces/ITScanRequestParams';

const DEFAULT_OPTIONS: ScanOptions = {
  keys: true,
  values: true,
  detectSentences: true,
  detectReservedWords: true,
};

const DEFAULT_SCAN_OPTIONS = {
  checkUrl: DEFAULT_OPTIONS,
  checkBody: DEFAULT_OPTIONS,
  checkHeaders: DEFAULT_OPTIONS,
  checkFormData: DEFAULT_OPTIONS,
  checkCookies: DEFAULT_OPTIONS,
};

/**
 * Escanea una solicitud HTTP en busca de posibles problemas de seguridad.
 *
 * @param params - Los parámetros de la solicitud a escanear.
 * @returns Un array de cadenas que representan los problemas de seguridad encontrados.
 *
 * @remarks
 * Esta función escanea una solicitud HTTP en busca de posibles problemas de seguridad en la URL, el cuerpo, los encabezados, los datos del formulario y las cookies.
 *
 * @param url - La URL de la solicitud.
 * @param body - El cuerpo de la solicitud.
 * @param headers - Los encabezados de la solicitud.
 * @param formData - Los datos del formulario de la solicitud.
 * @param cookies - Las cookies de la solicitud.
 * @param options - Las opciones de escaneo personalizadas.
 *
 * @returns Un array de cadenas que representan los problemas de seguridad encontrados.
 * @author Andres Gutierrez Velez
 * @since 21/06/2024
 * @contact sr.willardkraft@gmail.com
 */
export function scanRequest(params: ScanRequestParams): string[] {
  const {
    url = '',
    body = {},
    headers = {},
    formData = new FormData(),
    cookies = {},
    options = DEFAULT_SCAN_OPTIONS,
  } = params;

  const MERGED_OPTIONS = {
    checkUrl: { ...DEFAULT_OPTIONS, ...options.checkUrl },
    checkBody: { ...DEFAULT_OPTIONS, ...options.checkBody },
    checkHeaders: { ...DEFAULT_OPTIONS, ...options.checkHeaders },
    checkFormData: { ...DEFAULT_OPTIONS, ...options.checkFormData },
    checkCookies: { ...DEFAULT_OPTIONS, ...options.checkCookies },
  };

  let issues: string[] = [];

  if (url !== '') {
    const checkUrl = MERGED_OPTIONS.checkUrl;
    if (checkUrl.values) {
      issues = issues.concat(scanUrl(url, checkUrl));
    }
  }

  if (Object.keys(body).length > 0) {
    const checkBody = MERGED_OPTIONS.checkBody;
    issues = issues.concat(scanBody(body, checkBody));
  }

  if (Object.keys(headers).length > 0) {
    const checkHeaders = MERGED_OPTIONS.checkHeaders;
    issues = issues.concat(scanHeaders(headers, checkHeaders));
  }

  if (formData.entries().next().done === false) {
    const checkFormData = MERGED_OPTIONS.checkFormData;
    issues = issues.concat(scanFormData(formData, checkFormData));
  }

  if (Object.keys(cookies).length > 0) {
    const checkCookies = MERGED_OPTIONS.checkCookies;
    issues = issues.concat(scanCookies(cookies, checkCookies));
  }

  return issues;
}

const url =
  'http://example.com?username=dfgfd&name=JohnDoe&age=30&country=USA&email=john.doe@example.com&phone=555-1234&company=Tech%20Solutions%20Inc.&jobTitle=Senior%20Developer&param1=value1&param2=value2&param3=value3&param4=value4&param5=value5&param6=value6&param7=value7&param8=value8&param9=value9&param10=value10&param11=value11&param12=value12&param13=value13&param14=value14&param15=value15&param16=value16&param17=value17&param18=value18&param19=value19&param20=value20&param21=value21&param22=value22&param23=value23&param24=value24&param25=value25&param26=value26&param27=value27&param28=value28&param29=value29&param30=value30&param31=value31&param32=value32&param33=value33&param34=value34&param35=value35&param36=value36&param37=value37&param38=value38&param39=value39&param40=value40&param41=value41&param42=value42&param43=value43&param44=value44&param45=value45&param46=value46&param47=value47&param48=value48&param49=value49&param50=value50';

const body = {
  select: 'john_doe',
  selasdasect: 'select',
  username: 'john_doe',
  password: 'securePassword123',
  email: 'john.doe@example.com',
  address: '123 Main St, Springfield, USA',
  phone: '555-1234',
  bio: 'Software developer with 10 years of experience.',
  interests: 'coding, music, sports',
  company: 'Tech Solutions Inc.',
  jobTitle: 'Senior Developer',
  website: 'https://johndoe.com',
  github: 'https://github.com/johndoe',
  twitter: '@johndoe',
  linkedin: 'https://linkedin.com/in/johndoe',
  project: 'Open Source Contribution',
  skills: ['TypeScript', 'Nodg.js', 'React'],
  languages: ['English', 'Spanish'],
  certifications: [
    'AWS Certified Solutions Architect',
    'Certified Kubernetes Administrator',
  ],
  hobbies: 'hiking, reading, traveling',
  education: 'BSc in Computer Science',
  experience: [
    {
      role: 'Junior Developer',
      duration: '2 years',
    },
    {
      company: 'Another Company',
      role: 'Mid-Level Developer',
      duration: '3 years',
    },
  ],
  publications: [
    {
      urls: 'hf',
      url: 'https://example.com/js',
    },
    {
      title: 'Advanced TypeScript Techniques',
      url: 'https://example.com/ts',
    },
  ],
  awards: [
    {
      name: 'Developer of the Year',
      year: 2020,
    },
    {
      name: 'Best Open Source Contribution',
      year: 2021,
    },
  ],
  street: '456 Elm St',
  city: 'Metropolis',
  state: 'NY',
  zip: '10001',
  country: 'USA',
  dob: '1990-01-01',
  gender: 'male',
  maritalStatus: 'single',
  employmentStatus: 'employed',
  employer: 'Tech Corp',
  position: 'Software Engineer',
  salary: '100000',
  startDate: '2020-01-01',
  endDate: 'Present',
  project1: 'Project Alpha',
  project2: 'Project Beta',
  project3: 'Project Gamma',
  language1: 'English',
  language2: 'Spanish',
  language3: 'French',
  certification1: 'Certified JavaScript Developer',
  certification2: 'Certified Nodu.js Developer',
  certification3: 'Certified React Developer',
  hobby1: 'reading',
  hobby2: 'traveling',
  hobby3: 'swimming',
};

const headers = {
  'User-Agent': 'Mozilla/5.0',
  'Accept-Language': 'en-US,en;q=0.9',
  'Content-Type': 'text/html; charset=utf-8',
  Authorization: 'Bearer abcdef123456',
  Referer: 'https://example.com/form',
  'X-Requested-With': 'XMLHttpRequest',
  'X-Custom-Header1': 'CustomValue1',
  'X-Custom-Header2': 'CustomValue2',
  'X-Custom-Header3': 'CustomValue3',
  'X-Custom-Header4': 'CustomValue4',
  'X-Custom-Header5': 'CustomValue5',
  'X-Custom-Header6': 'CustomValue6',
  'X-Custom-Header7': 'CustomValue7',
  'X-Custom-Header8': 'CustomValue8',
  'X-Custom-Header9': 'CustomValue9',
  'X-Custom-Header10': 'CustomValue10',
  'X-Custom-Header11': 'CustomValue11',
  'X-Custom-Header12': 'CustomValue12',
  'X-Custom-Header13': 'CustomValue13',
  'X-Custom-Header14': 'CustomValue14',
  'X-Custom-Header15': 'CustomValue15',
  'X-Custom-Header16': 'CustomValue16',
  'X-Custom-Header17': 'CustomValue17',
  'X-Custom-Header18': 'CustomValue18',
  'X-Custom-Header19': 'CustomValue19',
  'X-Custom-Header20': 'CustomValue20',
  'X-Custom-Header21': 'CustomValue21',
  'X-Custom-Header22': 'CustomValue22',
  'X-Custom-Header23': 'CustomValue23',
  'X-Custom-Header24': 'CustomValue24',
  'X-Custom-Header25': 'CustomValue25',
  'X-Custom-Header26': 'CustomValue26',
  'X-Custom-Header27': 'CustomValue27',
  'X-Custom-Header28': 'CustomValue28',
  'X-Custom-Header29': 'CustomValue29',
  'X-Custom-Header30': 'CustomValue30',
  'X-Custom-Header31': 'CustomValue31',
  'X-Custom-Header32': 'CustomValue32',
  'X-Custom-Header33': 'CustomValue33',
  'X-Custom-Header34': 'CustomValue34',
  'X-Custom-Header35': 'CustomValue35',
  'X-Custom-Header36': 'CustomValue36',
  'X-Custom-Header37': 'CustomValue37',
  'X-Custom-Header38': 'CustomValue38',
  'X-Custom-Header39': 'CustomValue39',
  'X-Custom-Header40': 'CustomValue40',
  'X-Custom-Header41': 'CustomValue41',
  'X-Custom-Header42': 'CustomValue42',
  'X-Custom-Header43': 'CustomValue43',
  'X-Custom-Header44': 'CustomValue44',
  'X-Custom-Header45': 'CustomValue45',
  'X-Custom-Header46': 'CustomValue46',
  'X-Custom-Header47': 'CustomValue47',
  'X-Custom-Header48': 'CustomValue48',
  'X-Custom-Header49': 'CustomValue49',
  'X-Custom-Header50': 'CustomValue50',
};

const formData = new FormData();
formData.append('firstName', 'John');
formData.append('lastName', 'Doe');
formData.append('age', '30');
formData.append('email', 'john.doe@example.com');
formData.append('address', '123 Main St');
formData.append('city', 'Springfield');
formData.append('state', 'NY');
formData.append('zip', '10001');
formData.append('country', 'USA');
formData.append('phone', '555-1234');
formData.append('company', 'Tech Solutions Inc.');
formData.append('jobTitle', 'Senior Developer');
formData.append('bio', 'Software developer with 10 years of experience.');
formData.append('interests', 'coding, music, sports');
formData.append('project', 'Open Source Contribution');
formData.append('skills', 'JavaScript, TypeScript, Nodse.js, React');
formData.append('languages', 'English, Spanish');
formData.append('certifications', 'AWS Certified Solutions Architect');
formData.append('hobbies', 'hiking, reading, traveling');
formData.append('education', 'BSc in Computer Science');
formData.append('experience', 'Previous Company: Junior Developer, 2 years');
formData.append(
  'publications',
  'Introduction to Modern JavaScsripts: https://example.com/js',
);
formData.append('awards', 'Developer of the Year, 2020');
formData.append('project1', 'Project Alpha');
formData.append('project2', 'Project Beta');
formData.append('project3', 'Project Gamma');
formData.append('language1', 'English');
formData.append('language2', 'Spanish');
formData.append('language3', 'French');
formData.append('certification1', 'Certified JavaScript Developer');
formData.append('certification2', 'Certified Nodes.js Developer');
formData.append('certification3', 'Certified React Developer');
formData.append('hobby1', 'reading');
formData.append('hobby2', 'traveling');
formData.append('hobby3', 'swimming');
formData.append('field1', 'value1');
formData.append('field2', 'value2');
formData.append('field3', 'value3');
formData.append('field4', 'value4');
formData.append('field5', 'value5');
formData.append('field6', 'value6');
formData.append('field7', 'value7');
formData.append('field8', 'value8');
formData.append('field9', 'value9');
formData.append('field10', 'value10');
formData.append('field11', 'value11');
formData.append('field12', 'value12');
formData.append('field13', 'value13');
formData.append('field14', 'value14');
formData.append('field15', 'value15');
formData.append('field16', 'value16');
formData.append('field17', 'value17');
formData.append('field18', 'value18');
formData.append('field19', 'value19');
formData.append('field20', 'value20');
formData.append('field21', 'value21');
formData.append('field22', 'value22');
formData.append('field23', 'value23');
formData.append('field24', 'value24');
formData.append('field25', 'value25');
formData.append('field26', 'value26');
formData.append('field27', 'value27');
formData.append('field28', 'value28');
formData.append('field29', 'value29');
formData.append('field30', 'value30');
formData.append('field31', 'value31');
formData.append('field32', 'value32');
formData.append('field33', 'value33');
formData.append('field34', 'value34');
formData.append('field35', 'value35');
formData.append('field36', 'value36');
formData.append('field37', 'value37');
formData.append('field38', 'value38');
formData.append('field39', 'value39');
formData.append('field40', 'value40');
formData.append('field41', 'value41');
formData.append('field42', 'value42');
formData.append('field43', 'value43');
formData.append('field44', 'value44');
formData.append('field45', 'value45');
formData.append('field46', 'value46');
formData.append('field47', 'value47');
formData.append('field48', 'value48');
formData.append('field49', 'value49');
formData.append('field50', 'value50');
formData.append(
  'profilePicture',
  new Blob([''], { type: 'image/png' }),
  'profile.png',
);
formData.append(
  'resume',
  new Blob([''], { type: 'application/pdf' }),
  'resume.pdf',
);
formData.append(
  'coverLetter',
  new Blob([''], { type: 'application/msword' }),
  'cover_letter.doc',
);
formData.append(
  'portfolio',
  new Blob([''], { type: 'application/zip' }),
  'portfolio.zip',
);

const cookies = {
  sessionId: 'abcd1234',
  theme: 'dark',
  language: 'en',
  authToken: 'xyz7890',
  trackingId: 'track1234',
  preferences: 'safeMode=true; fontSize=medium',
  cookie1: 'value1',
  cookie2: 'value2',
  cookie3: 'value3',
  cookie4: 'value4',
  cookie5: 'value5',
  cookie6: 'value6',
  cookie7: 'value7',
  cookie8: 'value8',
  cookie9: 'value9',
  cookie10: 'value10',
  cookie11: 'value11',
  cookie12: 'value12',
  cookie13: 'value13',
  cookie14: 'value14',
  cookie15: 'value15',
  cookie16: 'value16',
  cookie17: 'value17',
  cookie18: 'value18',
  cookie19: 'value19',
  cookie20: 'value20',
  cookie21: 'value21',
  cookie22: 'value22',
  cookie23: 'value23',
  cookie24: 'value24',
  cookie25: 'value25',
  cookie26: 'value26',
  cookie27: 'value27',
  cookie28: 'value28',
  cookie29: 'value29',
  cookie30: 'value30',
  cookie31: 'value31',
  cookie32: 'value32',
  cookie33: 'value33',
  cookie34: 'value34',
  cookie35: 'value35',
  cookie36: 'value36',
  cookie37: 'value37',
  cookie38: 'value38',
  cookie39: 'value39',
  cookie40: 'value40',
  cookie41: 'value41',
  cookie42: 'value42',
  cookie43: 'value43',
  cookie44: 'value44',
  cookie45: 'value45',
  cookie46: 'value46',
  cookie47: 'value47',
  cookie48: 'value48',
  cookie49: 'value49',
  cookie50: 'value50',
};

const scanOptions = {
  checkUrl: {
    keys: false,
    values: true,
    detectSentences: true,
    detectReservedWords: true,
  },
  checkBody: {
    keys: true,
    values: true,
    detectSentences: true,
    detectReservedWords: true,
  },
  checkHeaders: {
    keys: true,
    values: true,
    detectSentences: true,
    detectReservedWords: true,
  },
  checkFormData: {
    keys: true,
    values: true,
    detectSentences: true,
    detectReservedWords: true,
  },
  checkCookies: {
    keys: true,
    values: true,
    detectSentences: true,
    detectReservedWords: true,
  },
};

const issues = scanRequest({
  url,
  body,
  headers,
  formData,
  cookies,
  options: scanOptions,
});

if (issues.length > 0) {
  console.log('Potential security issues detected:');
  issues.forEach((issue) => console.log(issue));
} else {
  console.log('No issues detected.');
}
